from ..common import sessions_common_pb2 as _sessions_common_pb2
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor



#!/usr/bin/env python3
import pytest

from google.protobuf.timestamp_pb2 import Timestamp
from google.protobuf.duration_pb2 import Duration
from dataclasses import dataclass
from datetime import datetime, timedelta, timezone
from armonik.common.helpers import (
    datetime_to_timestamp,
    timestamp_to_datetime,
    timedelta_to_duration,
    duration_to_timedelta,
    batched,
)

from typing import Iterable, List


@dataclass
class Case:
    seconds: int
    nanos: int

    @property
    def timestamp(self) -> Timestamp:
        return Timestamp(seconds=self.seconds, nanos=self.nanos)

    @property
    def date_time(self) -> datetime:
        return datetime.utcfromtimestamp(self.seconds + self.nanos * 1e-9).replace(
            tzinfo=timezone.utc
        )

    @property
    def duration(self) -> Duration:
        return Duration(seconds=self.seconds, nanos=self.nanos)

    @property
    def delta(self) -> timedelta:
        return timedelta(seconds=self.seconds, microseconds=self.nanos / 1000)


test_cases = [Case(1234, 1234), Case(0, 0), Case(12345, 12345), Case(100000, 100000)]


@pytest.mark.parametrize("case", test_cases)
def test_datetime_to_timestamp(case: Case):
    ts = datetime_to_timestamp(case.date_time)
    assert ts.seconds == case.timestamp.seconds and abs(ts.nanos - case.timestamp.nanos) < 1000


@pytest.mark.parametrize("case", test_cases)
def test_timestamp_to_datetime(case: Case):
    ts = timestamp_to_datetime(case.timestamp)
    assert ts.timestamp() == case.date_time.timestamp()


@pytest.mark.parametrize("case", test_cases)
def test_duration_to_timedelta(case: Case):
    ts = duration_to_timedelta(case.duration)
    assert ts.total_seconds() == case.delta.total_seconds()


@pytest.mark.parametrize("case", test_cases)
def test_timedelta_to_duration(case: Case):
    ts = timedelta_to_duration(case.delta)
    assert ts.seconds == case.duration.seconds and abs(ts.nanos - case.duration.nanos) < 1000


@pytest.mark.parametrize(
    ["iterable", "batch_size", "iterations"],
    [
        ([1, 2, 3], 3, [[1, 2, 3]]),
        ([1, 2, 3], 5, [[1, 2, 3]]),
        ([1, 2, 3], 2, [[1, 2], [3]]),
        (
            [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
            3,
            [[1, 2, 3], [4, 5, 6], [7, 8, 9], [10, 11]],
        ),
    ],
)
def test_batched(iterable: Iterable, batch_size: int, iterations: List[Iterable]):
    for index, batch in enumerate(batched(iterable, batch_size)):
        assert batch == iterations[index]



import datetime
import pytest
import warnings

from .conftest import all_rpc_called, rpc_called, get_client
from armonik.client import ArmoniKResults, ResultFieldFilter
from armonik.common import Result, ResultStatus


class TestArmoniKResults:
    def test_get_result(self):
        results_client: ArmoniKResults = get_client("Results")
        result = results_client.get_result("result-name")

        assert rpc_called("Results", "GetResult")
        assert isinstance(result, Result)
        assert result.session_id == "session-id"
        assert result.name == "result-name"
        assert result.owner_task_id == "owner-task-id"
        assert result.status == 2
        assert result.created_at == datetime.datetime(
            1970, 1, 1, 0, 0, tzinfo=datetime.timezone.utc
        )
        assert result.completed_at == datetime.datetime(
            1970, 1, 1, 0, 0, tzinfo=datetime.timezone.utc
        )
        assert result.result_id == "result-id"
        assert result.size == 0

    def test_get_owner_task_id(self):
        results_client: ArmoniKResults = get_client("Results")
        results_tasks = results_client.get_owner_task_id(["result-id"], "session-id")

        assert rpc_called("Results", "GetOwnerTaskId")
        # TODO: Mock must be updated to return something and so that changes the following assertions
        assert results_tasks == {}

    def test_list_results_no_filter(self):
        results_client: ArmoniKResults = get_client("Results")
        num, results = results_client.list_results()

        assert rpc_called("Results", "ListResults")
        # TODO: Mock must be updated to return something and so that changes the following assertions
        assert num == 0
        assert results == []

    def test_list_results_with_filter(self):
        results_client: ArmoniKResults = get_client("Results")
        num, results = results_client.list_results(
            ResultFieldFilter.STATUS == ResultStatus.COMPLETED
        )

        assert rpc_called("Results", "ListResults", 2)
        # TODO: Mock must be updated to return something and so that changes the following assertions
        assert num == 0
        assert results == []

    def test_create_results_metadata(self):
        results_client: ArmoniKResults = get_client("Results")
        results = results_client.create_results_metadata(["result-name"], "session-id")

        assert rpc_called("Results", "CreateResultsMetaData")
        # TODO: Mock must be updated to return something and so that changes the following assertions
        assert results == {}

    def test_create_results(self):
        results_client: ArmoniKResults = get_client("Results")
        results = results_client.create_results({"result-name": b"test data"}, "session-id")

        assert rpc_called("Results", "CreateResults")
        assert results == {}

    def test_get_service_config(self):
        results_client: ArmoniKResults = get_client("Results")
        chunk_size = results_client.get_service_config()

        assert rpc_called("Results", "GetServiceConfiguration")
        assert isinstance(chunk_size, int)
        assert chunk_size == 81920

    def test_upload_result_data(self):
        results_client: ArmoniKResults = get_client("Results")
        result = results_client.upload_result_data("result-name", "session-id", b"test data")

        assert rpc_called("Results", "UploadResultData")
        assert result is None

    def test_download_result_data(self):
        results_client: ArmoniKResults = get_client("Results")
        data = results_client.download_result_data("result-name", "session-id")

        assert rpc_called("Results", "DownloadResultData")
        assert data == b""

    def test_delete_result_data(self):
        results_client: ArmoniKResults = get_client("Results")
        result = results_client.delete_result_data(["result-name"], "session-id")

        assert rpc_called("Results", "DeleteResultsData")
        assert result is None

    def test_watch_results(self):
        results_client: ArmoniKResults = get_client("Results")
        with pytest.raises(NotImplementedError, match=""):
            results_client.watch_results()
        assert rpc_called("Results", "WatchResults", 0)

    def test_get_results_ids(self):
        with warnings.catch_warnings(record=True) as w:
            # Cause all warnings to always be triggered.
            warnings.simplefilter("always")

            results_client: ArmoniKResults = get_client("Results")
            results = results_client.get_results_ids("session-id", ["result_1"])

            assert issubclass(w[-1].category, DeprecationWarning)
            assert rpc_called("Results", "CreateResultsMetaData", 2)
            assert results == {}

    def test_service_fully_implemented(self):
        assert all_rpc_called("Results", missings=["WatchResults"])



from .conftest import all_rpc_called, rpc_called, get_client
from armonik.client import ArmoniKHealthChecks
from armonik.common import ServiceHealthCheckStatus


class TestArmoniKHealthChecks:
    def test_check_health(self):
        health_checks_client: ArmoniKHealthChecks = get_client("HealthChecks")
        services_health = health_checks_client.check_health()

        assert rpc_called("HealthChecks", "CheckHealth")
        assert services_health == {
            "mock": {
                "message": "Mock is healthy",
                "status": ServiceHealthCheckStatus.HEALTHY,
            }
        }

    def test_service_fully_implemented(self):
        assert all_rpc_called("HealthChecks")



from __future__ import annotations

from typing import Dict, List, Tuple, cast

from deprecation import deprecated
from grpc import Channel

from ..common import Direction, Result
from ..common.filter import Filter, StatusFilter, StringFilter
from ..common.helpers import batched
from ..protogen.client.results_service_pb2_grpc import ResultsStub
from ..protogen.common.objects_pb2 import Empty
from ..protogen.common.results_common_pb2 import (
    CreateResultsMetaDataRequest,
    CreateResultsMetaDataResponse,
    CreateResultsRequest,
    CreateResultsResponse,
    DeleteResultsDataRequest,
    DownloadResultDataRequest,
    GetOwnerTaskIdRequest,
    GetOwnerTaskIdResponse,
    GetResultRequest,
    GetResultResponse,
    ListResultsRequest,
    ListResultsResponse,
    ResultsServiceConfigurationResponse,
    UploadResultDataRequest,
)
from ..protogen.common.results_fields_pb2 import (
    RESULT_RAW_ENUM_FIELD_RESULT_ID,
    RESULT_RAW_ENUM_FIELD_STATUS,
    ResultField,
    ResultRawField,
)
from ..protogen.common.results_filters_pb2 import (
    FilterField as rawFilterField,
)
from ..protogen.common.results_filters_pb2 import (
    Filters as rawFilters,
)
from ..protogen.common.results_filters_pb2 import (
    FiltersAnd as rawFilterAnd,
)
from ..protogen.common.results_filters_pb2 import (
    FilterStatus as rawFilterStatus,
)
from ..protogen.common.sort_direction_pb2 import SortDirection


class ResultFieldFilter:
    STATUS = StatusFilter(
        ResultField(result_raw_field=ResultRawField(field=RESULT_RAW_ENUM_FIELD_STATUS)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
        rawFilterStatus,
    )
    RESULT_ID = StringFilter(
        ResultField(result_raw_field=ResultRawField(field=RESULT_RAW_ENUM_FIELD_RESULT_ID)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
    )


class ArmoniKResults:
    def __init__(self, grpc_channel: Channel):
        """Result service client

        Args:
            grpc_channel: gRPC channel to use
        """
        self._client = ResultsStub(grpc_channel)

    @deprecated(
        deprecated_in="3.15.0",
        details="Use create_result_metadata or create_result insted.",
    )
    def get_results_ids(self, session_id: str, names: List[str]) -> Dict[str, str]:
        return {
            r.name: r.result_id
            for r in cast(
                CreateResultsMetaDataResponse,
                self._client.CreateResultsMetaData(
                    CreateResultsMetaDataRequest(
                        results=[CreateResultsMetaDataRequest.ResultCreate(name=n) for n in names],
                        session_id=session_id,
                    )
                ),
            ).results
        }

    def list_results(
        self,
        result_filter: Filter | None = None,
        page: int = 0,
        page_size: int = 1000,
        sort_field: Filter = ResultFieldFilter.STATUS,
        sort_direction: SortDirection = Direction.ASC,
    ) -> Tuple[int, List[Result]]:
        """List results based on a filter.

        Args:
            result_filter (Filter): Filter to apply when listing results
            page: page number to request, useful for pagination, defaults to 0
            page_size: size of a page, defaults to 1000
            sort_field: field to sort the resulting list by, defaults to the status
            sort_direction: direction of the sort, defaults to ascending
        Returns:
            A tuple containing :
            - The total number of results for the given filter
            - The obtained list of results
        """
        request: ListResultsRequest = ListResultsRequest(
            page=page,
            page_size=page_size,
            filters=cast(rawFilters, result_filter.to_disjunction().to_message())
            if result_filter
            else None,
            sort=ListResultsRequest.Sort(
                field=cast(ResultField, sort_field.field), direction=sort_direction
            ),
        )
        list_response: ListResultsResponse = self._client.ListResults(request)
        return list_response.total, [Result.from_message(r) for r in list_response.results]

    def get_result(self, result_id: str) -> Result:
        """Get a result by id.

        Args:
            result_id: The ID of the result.

        Return:
            The result summary.
        """
        request = GetResultRequest(result_id=result_id)
        response: GetResultResponse = self._client.GetResult(request)
        return Result.from_message(response.result)

    def get_owner_task_id(
        self, result_ids: List[str], session_id: str, batch_size: int = 500
    ) -> Dict[str, str]:
        """Get the IDs of the tasks that should produce the results.

        Args:
            result_ids: A list of results.
            session_id: The ID of the session to which the results belongs.
            batch_size: Batch size for querying.

        Return:
            A dictionnary mapping results to owner task ID.
        """
        results = {}
        for result_ids_batch in batched(result_ids, batch_size):
            request = GetOwnerTaskIdRequest(session_id=session_id, result_id=result_ids_batch)
            response: GetOwnerTaskIdResponse = self._client.GetOwnerTaskId(request)
            for result_task in response.result_task:
                results[result_task.result_id] = result_task.task_id
        return results

    def create_results_metadata(
        self, result_names: List[str], session_id: str, batch_size: int = 100
    ) -> Dict[str, Result]:
        """Create the metadata of multiple results at once.
        Data have to be uploaded separately.

        Args:
            result_names: The list of the names of the results to create.
            session_id: The ID of the session to which the results belongs.
            batch_size: Batch size for querying.

        Return:
            A dictionnary mapping each result name to its corresponding result summary.
        """
        results = {}
        for result_names_batch in batched(result_names, batch_size):
            request = CreateResultsMetaDataRequest(
                results=[
                    CreateResultsMetaDataRequest.ResultCreate(name=result_name)
                    for result_name in result_names_batch
                ],
                session_id=session_id,
            )
            response: CreateResultsMetaDataResponse = self._client.CreateResultsMetaData(request)
            for result_message in response.results:
                results[result_message.name] = Result.from_message(result_message)
        return results

    def create_results(
        self, results_data: Dict[str, bytes], session_id: str, batch_size: int = 1
    ) -> Dict[str, Result]:
        """Create one result with data included in the request.

        Args:
            results_data: A dictionnary mapping the result names to their actual data.
            session_id: The ID of the session to which the results belongs.
            batch_size: Batch size for querying.

        Return:
            A dictionnary mappin each result name to its corresponding result summary.
        """
        results = {}
        for results_names_batch in batched(results_data.keys(), batch_size):
            request = CreateResultsRequest(
                results=[
                    CreateResultsRequest.ResultCreate(name=name, data=results_data[name])
                    for name in results_names_batch
                ],
                session_id=session_id,
            )
            response: CreateResultsResponse = self._client.CreateResults(request)
            for message in response.results:
                results[message.name] = Result.from_message(message)
        return results

    def upload_result_data(
        self, result_id: str, session_id: str, result_data: bytes | bytearray
    ) -> None:
        """Upload data for an empty result already created.

        Args:
            result_id: The ID of the result.
            result_data: The result data.
            session_id: The ID of the session.
        """
        data_chunk_max_size = self.get_service_config()

        def upload_result_stream():
            request = UploadResultDataRequest(
                id=UploadResultDataRequest.ResultIdentifier(
                    session_id=session_id, result_id=result_id
                )
            )
            yield request

            start = 0
            data_len = len(result_data)
            while start < data_len:
                chunk_size = min(data_chunk_max_size, data_len - start)
                request = UploadResultDataRequest(
                    data_chunk=result_data[start : start + chunk_size]
                )
                yield request
                start += chunk_size

        self._client.UploadResultData(upload_result_stream())

    def download_result_data(self, result_id: str, session_id: str) -> bytes:
        """Retrieve data of a result.

        Args:
            result_id: The ID of the result.
            session_id: The session of the result.

        Return:
            Result data.
        """
        request = DownloadResultDataRequest(result_id=result_id, session_id=session_id)
        streaming_call = self._client.DownloadResultData(request)
        return b"".join([message.data_chunk for message in streaming_call])

    def delete_result_data(
        self, result_ids: List[str], session_id: str, batch_size: int = 100
    ) -> None:
        """Delete data from multiple results

        Args:
            result_ids: The IDs of the results which data must be deleted.
            session_id: The ID of the session to which the results belongs.
            batch_size: Batch size for querying.
        """
        for result_ids_batch in batched(result_ids, batch_size):
            request = DeleteResultsDataRequest(result_id=result_ids_batch, session_id=session_id)
            self._client.DeleteResultsData(request)

    def get_service_config(self) -> int:
        """Get the configuration of the service.

        Return:
            Maximum size supported by a data chunk for the result service.
        """
        response: ResultsServiceConfigurationResponse = self._client.GetServiceConfiguration(
            Empty()
        )
        return response.data_chunk_max_size

    def watch_results(self):
        raise NotImplementedError()



import datetime

from .conftest import all_rpc_called, rpc_called, get_client
from armonik.client import ArmoniKTasks, TaskFieldFilter
from armonik.common import Task, TaskDefinition, TaskOptions, TaskStatus, Output


class TestArmoniKTasks:
    mock_task = Task(
        id="task-id",
        session_id="session-id",
        owner_pod_id="",
        parent_task_ids=[],
        data_dependencies=[],
        expected_output_ids=[],
        retry_of_ids=[],
        status=4,
        payload_id=None,
        status_message="",
        options=TaskOptions(
            max_duration=datetime.timedelta(seconds=1),
            priority=1,
            max_retries=1,
            partition_id="partition-id",
            application_name="application-name",
            application_version="application-version",
            application_namespace="application-namespace",
            application_service="application-service",
            engine_type="engine-type",
            options={},
        ),
        created_at=datetime.datetime(1970, 1, 1, 0, 0, tzinfo=datetime.timezone.utc),
        submitted_at=datetime.datetime(1970, 1, 1, 0, 0, tzinfo=datetime.timezone.utc),
        started_at=datetime.datetime(1970, 1, 1, 0, 0, tzinfo=datetime.timezone.utc),
        ended_at=datetime.datetime(1970, 1, 1, 0, 0, tzinfo=datetime.timezone.utc),
        pod_ttl=datetime.datetime(1970, 1, 1, 0, 0, tzinfo=datetime.timezone.utc),
        output=Output(error=""),
        pod_hostname="",
        received_at=datetime.datetime(1970, 1, 1, 0, 0, tzinfo=datetime.timezone.utc),
        acquired_at=datetime.datetime(1970, 1, 1, 0, 0, tzinfo=datetime.timezone.utc),
    )

    def test_get_task(self):
        tasks_client: ArmoniKTasks = get_client("Tasks")
        task = tasks_client.get_task("task-id")

        assert rpc_called("Tasks", "GetTask")
        assert isinstance(task, Task)
        assert task == self.mock_task

    def test_list_tasks_detailed_no_filter(self):
        tasks_client: ArmoniKTasks = get_client("Tasks")
        num, tasks = tasks_client.list_tasks()
        assert rpc_called("Tasks", "ListTasksDetailed")
        # TODO: Mock must be updated to return something and so that changes the following assertions
        assert num == 1
        assert tasks == [self.mock_task]

    def test_list_tasks_detailed_with_filter(self):
        tasks_client: ArmoniKTasks = get_client("Tasks")
        num, tasks = tasks_client.list_tasks(TaskFieldFilter.STATUS == TaskStatus.COMPLETED)
        assert rpc_called("Tasks", "ListTasksDetailed", 2)
        # TODO: Mock must be updated to return something and so that changes the following assertions
        assert num == 1
        assert tasks == [self.mock_task]

    def test_list_tasks_no_detailed_no_filter(self):
        tasks_client: ArmoniKTasks = get_client("Tasks")
        num, tasks = tasks_client.list_tasks(detailed=False)
        assert rpc_called("Tasks", "ListTasks")
        # TODO: Mock must be updated to return something and so that changes the following assertions
        assert num == 1
        assert tasks == []

    def test_cancel_tasks(self):
        tasks_client: ArmoniKTasks = get_client("Tasks")
        tasks = tasks_client.cancel_tasks(["task-id-1", "task-id-2"])

        assert rpc_called("Tasks", "CancelTasks")
        assert tasks is None

    def test_get_result_ids(self):
        tasks_client: ArmoniKTasks = get_client("Tasks")
        tasks_results = tasks_client.get_result_ids(["task-id-1", "task-id-2"])
        assert rpc_called("Tasks", "GetResultIds")
        # TODO: Mock must be updated to return something and so that changes the following assertions
        assert tasks_results == {}

    def test_count_tasks_by_status_no_filter(self):
        tasks_client: ArmoniKTasks = get_client("Tasks")
        count = tasks_client.count_tasks_by_status()
        assert rpc_called("Tasks", "CountTasksByStatus")
        # TODO: Mock must be updated to return something and so that changes the following assertions
        assert count == {TaskStatus.COMPLETED: 2, TaskStatus.SUBMITTED: 5}

    def test_count_tasks_by_status_with_filter(self):
        tasks_client: ArmoniKTasks = get_client("Tasks")
        count = tasks_client.count_tasks_by_status(TaskFieldFilter.STATUS == TaskStatus.COMPLETED)
        assert rpc_called("Tasks", "CountTasksByStatus", 2)
        # TODO: Mock must be updated to return something and so that changes the following assertions
        assert count == {TaskStatus.COMPLETED: 2, TaskStatus.SUBMITTED: 5}

    def test_submit_tasks(self):
        tasks_client: ArmoniKTasks = get_client("Tasks")
        tasks = tasks_client.submit_tasks(
            "session-id",
            [
                TaskDefinition(
                    payload_id="payload-id",
                    expected_output_ids=["result-id"],
                    data_dependencies=[],
                    options=TaskOptions(
                        max_duration=datetime.timedelta(seconds=1),
                        priority=1,
                        max_retries=1,
                    ),
                )
            ],
            default_task_options=TaskOptions(
                max_duration=datetime.timedelta(seconds=1), priority=1, max_retries=1
            ),
        )
        assert rpc_called("Tasks", "SubmitTasks")
        # TODO: Mock must be updated to return something and so that changes the following assertions
        assert tasks is None

    def test_service_fully_implemented(self):
        assert all_rpc_called("Tasks")



from __future__ import annotations
import logging
import sys
import json
import traceback
from logging import getLogger
from datetime import datetime
from types import TracebackType
from typing import Dict, Optional, Union, Tuple, Type


class ClefLogger:
    _worker_loggers: Dict[str, "ClefLogger"] = {}

    @classmethod
    def setup_logging(cls, minlevel: int = logging.INFO) -> None:
        """Activates logging, should only be done once per process

        Args:
            minlevel: Minimum level of logging for the whole process
        """
        logging.basicConfig(level=minlevel, stream=sys.stdout, format="%(message)s")

    @classmethod
    def getLogger(cls, name: str) -> "ClefLogger":
        """Get the logger with the given name. Creates it if it doesn't exist

        Args:
            name: Name of the logger

        Returns:
            ClefLogger logger of the requested name
        """
        if name in ClefLogger._worker_loggers:
            return ClefLogger._worker_loggers[name]
        ClefLogger._worker_loggers[name] = cls(name)
        return ClefLogger._worker_loggers[name]

    def __init__(self, name: Optional[str] = None, level: int = logging.INFO):
        self._logger = getLogger(name)
        self._logger.setLevel(level)

    def info(self, message: str, **kwargs):
        """Log a message at the info level

        Args:
            message: Message content, can contain '{name}' where name is a keyword argument given to this function (see kwargs)
            **kwargs: Keyword arguments added to the record
        """
        self.log(logging.INFO, message, **kwargs)

    def debug(self, message: str, **kwargs):
        """Log a message at the debug level

        Args:
            message: Message content, can contain '{name}' where name is a keyword argument given to this function (see kwargs)
            **kwargs: Keyword arguments added to the record
        """
        self.log(logging.DEBUG, message, **kwargs)

    def warning(self, message: str, **kwargs):
        """Log a message at the warning level

        Args:
            message: Message content, can contain '{name}' where name is a keyword argument given to this function (see kwargs)
            **kwargs: Keyword arguments added to the record
        """
        self.log(logging.WARNING, message, **kwargs)

    def error(self, message: str, **kwargs):
        """Log a message at the error level

        Args:
            message: Message content, can contain '{name}' where name is a keyword argument given to this function (see kwargs)
            **kwargs: Keyword arguments added to the record
        """
        self.log(logging.ERROR, message, **kwargs)

    def critical(self, message: str, **kwargs):
        """Log a message at the critical level

        Args:
            message: Message content, can contain '{name}' where name is a keyword argument given to this function (see kwargs)
            **kwargs: Keyword arguments added to the record
        """
        self.log(logging.CRITICAL, message, **kwargs)

    def exception(
        self,
        message: str,
        exc_info: Union[
            BaseException,
            Tuple[Type[BaseException], BaseException, Optional[TracebackType]],
            bool,
            None,
        ] = None,
        **kwargs,
    ):
        """Log a message at the error level with an optional exc_info

        Args:
            message: Message content, can contain '{name}' where name is a keyword argument given to this function (see kwargs)
            exc_info: Optional exc_info to add to an exception record, can be an exception, a boolean (to add the result of sys.exc_info() automatically if true), or the tuple given by sys.exc_info()
            **kwargs: Keyword arguments added to the record
        """
        self.log(logging.ERROR, message, exc_info=exc_info, **kwargs)

    def log(
        self,
        level: int,
        message: str,
        exc_info: Union[
            BaseException,
            Union[
                Tuple[
                    Union[Type[BaseException], None],
                    Union[BaseException, None],
                    Optional[TracebackType],
                ],
                None,
            ],
            bool,
            None,
        ] = None,
        **kwargs,
    ):
        """Log a message

        Args:
            level: level of the message
            message: Message content, can contain '{name}' where name is an keyword argument given to this function (see kwargs)
            exc_info: Optional exc_info to add to an exception record, can be an exception, a boolean (to add the result of sys.exc_info() automatically if true), or the tuple given by sys.exc_info()
            **kwargs: Keyword arguments added to the record
        """
        try:
            if self._logger.isEnabledFor(level):
                payload = {
                    "@t": datetime.utcnow().isoformat(),
                    "@l": logging.getLevelName(level),
                    "@mt": message,
                }
                if exc_info:
                    if isinstance(exc_info, bool):
                        exc_info = sys.exc_info()
                    elif isinstance(exc_info, BaseException):
                        exc_info = (type(exc_info), exc_info, exc_info.__traceback__)
                    payload["@x"] = "\n".join(traceback.format_exception(*exc_info))
                for k, v in kwargs:
                    if k.startswith("@"):
                        k = "@" + k
                    payload[k] = str(v)
                self._logger.log(level, msg=json.dumps(payload))
        except Exception as e:
            print(f"Couldn't log message : {e}")

    def setLevel(self, level: int):
        """Sets the level of this logger

        Args:
            level: Logging level
        """
        self._logger.setLevel(level)



from __future__ import annotations

from typing import List, Optional, Tuple, cast

from grpc import Channel

from ..common import Direction, Session, TaskOptions
from ..common.filter import Filter, StatusFilter, StringFilter
from ..protogen.client.sessions_service_pb2_grpc import SessionsStub
from ..protogen.common.sessions_common_pb2 import (
    CancelSessionRequest,
    CreateSessionRequest,
    GetSessionRequest,
    GetSessionResponse,
    ListSessionsRequest,
    ListSessionsResponse,
)
from ..protogen.common.sessions_fields_pb2 import (
    SESSION_RAW_ENUM_FIELD_STATUS,
    SessionField,
    SessionRawField,
    TaskOptionGenericField,
)
from ..protogen.common.sessions_filters_pb2 import (
    FilterField as rawFilterField,
)
from ..protogen.common.sessions_filters_pb2 import (
    Filters as rawFilters,
)
from ..protogen.common.sessions_filters_pb2 import (
    FiltersAnd as rawFilterAnd,
)
from ..protogen.common.sessions_filters_pb2 import (
    FilterStatus as rawFilterStatus,
)
from ..protogen.common.sort_direction_pb2 import SortDirection


class SessionFieldFilter:
    """
    Enumeration of the available filters
    """

    STATUS = StatusFilter(
        SessionField(session_raw_field=SessionRawField(field=SESSION_RAW_ENUM_FIELD_STATUS)),
        rawFilters,
        rawFilterAnd,
        rawFilterField,
        rawFilterStatus,
    )

    @staticmethod
    def task_options_key(option_key: str) -> StringFilter:
        """
        Filter for the TaskOptions.Options dictionary
        Args:
            option_key: key in the dictionary

        Returns:
            Corresponding filter
        """
        return StringFilter(
            SessionField(task_option_generic_field=TaskOptionGenericField(field=option_key)),
            rawFilters,
            rawFilterAnd,
            rawFilterField,
        )


class ArmoniKSessions:
    def __init__(self, grpc_channel: Channel):
        """Session service client

        Args:
            grpc_channel: gRPC channel to use
        """
        self._client = SessionsStub(grpc_channel)

    def create_session(
        self,
        default_task_options: TaskOptions,
        partition_ids: Optional[List[str]] = None,
    ) -> str:
        """Create a session

        Args:
            default_task_options: Default TaskOptions used when
                submitting tasks without specifying the options
            partition_ids: List of partitions this session can send
                tasks to. If unspecified, can only send to the default
                partition

        Returns:
            Session Id
        """
        request = CreateSessionRequest(
            default_task_option=default_task_options.to_message(),
            partition_ids=partition_ids if partition_ids else [],
        )
        return self._client.CreateSession(request).session_id

    def get_session(self, session_id: str):
        """Get a session by its ID.

        Args:
            session_id: The ID of the session.

        Return:
            The session summary.
        """
        request = GetSessionRequest(session_id=session_id)
        response: GetSessionResponse = self._client.GetSession(request)
        return Session.from_message(response.session)

    def list_sessions(
        self,
        session_filter: Filter | None = None,
        page: int = 0,
        page_size: int = 1000,
        sort_field: Filter = SessionFieldFilter.STATUS,
        sort_direction: SortDirection = Direction.ASC,
    ) -> Tuple[int, List[Session]]:
        """
        List sessions

        Args:
            session_filter (Filter): Filter to apply when listing sessions.
            page: page number to request, this can be useful when paginating the result, defaults to 0
            page_size: size of a page, defaults to 1000
            sort_field: field on which to sort the resulting list, defaults to the status
            sort_direction: direction of the sort, defaults to ascending

        Returns:
            A tuple containing :
            - The total number of sessions for the given filter
            - The obtained list of sessions
        """
        request = ListSessionsRequest(
            page=page,
            page_size=page_size,
            filters=cast(rawFilters, session_filter.to_disjunction().to_message())
            if session_filter
            else None,
            sort=ListSessionsRequest.Sort(
                field=cast(SessionField, sort_field.field), direction=sort_direction
            ),
        )
        response: ListSessionsResponse = self._client.ListSessions(request)
        return response.total, [Session.from_message(s) for s in response.sessions]

    def cancel_session(self, session_id: str) -> None:
        """Cancel a session

        Args:
            session_id: Id of the session to b cancelled
        """
        self._client.CancelSession(CancelSessionRequest(session_id=session_id))



from __future__ import annotations
import os
from deprecation import deprecated
from typing import Optional, Dict, List, Tuple

from ..common import TaskOptions, TaskDefinition, Task, Result
from ..protogen.common.agent_common_pb2 import (
    CreateTaskRequest,
    CreateResultsMetaDataRequest,
    CreateResultsMetaDataResponse,
    NotifyResultDataRequest,
    CreateResultsRequest,
    CreateResultsResponse,
    SubmitTasksRequest,
)
from ..protogen.common.objects_pb2 import (
    TaskRequest,
    DataChunk,
    InitTaskRequest,
    TaskRequestHeader,
    Configuration,
)
from ..protogen.worker.agent_service_pb2_grpc import AgentStub
from ..protogen.common.worker_common_pb2 import ProcessRequest
from ..common.helpers import batched


class TaskHandler:
    def __init__(self, request: ProcessRequest, agent_client: AgentStub):
        self._client: AgentStub = agent_client
        self.session_id: str = request.session_id
        self.task_id: str = request.task_id
        self.task_options: TaskOptions = TaskOptions.from_message(request.task_options)
        self.token: str = request.communication_token
        self.expected_results: List[str] = list(request.expected_output_keys)
        self.configuration: Configuration = request.configuration
        self.payload_id: str = request.payload_id
        self.data_folder: str = request.data_folder

        # TODO: Lazy load
        with open(os.path.join(self.data_folder, self.payload_id), "rb") as f:
            self.payload = f.read()

        # TODO: Lazy load
        self.data_dependencies: Dict[str, bytes] = {}
        for dd in request.data_dependencies:
            with open(os.path.join(self.data_folder, dd), "rb") as f:
                self.data_dependencies[dd] = f.read()

    @deprecated(
        deprecated_in="3.15.0",
        details="Use submit_tasks and instead and create the payload using create_result_metadata and send_result",
    )
    def create_tasks(
        self, tasks: List[TaskDefinition], task_options: Optional[TaskOptions] = None
    ) -> Tuple[List[Task], List[str]]:
        """Create new tasks for ArmoniK

        Args:
            tasks: List of task definitions
            task_options: Task Options used for this batch of tasks

        Returns:
            Tuple containing the list of successfully sent tasks, and
            the list of submission errors if any
        """
        task_requests = []

        for t in tasks:
            task_request = TaskRequest()
            task_request.expected_output_keys.extend(t.expected_output_ids)
            task_request.data_dependencies.extend(t.data_dependencies)
            task_request.payload = t.payload
            task_requests.append(task_request)
        assert self.configuration is not None
        create_tasks_reply = self._client.CreateTask(
            _to_request_stream(
                task_requests,
                self.token,
                task_options.to_message() if task_options is not None else None,
                self.configuration.data_chunk_max_size,
            )
        )
        ret = create_tasks_reply.WhichOneof("Response")
        if ret is None or ret == "error":
            raise Exception(f"Issue with server when submitting tasks : {create_tasks_reply.error}")
        elif ret == "creation_status_list":
            tasks_created = []
            tasks_creation_failed = []
            for creation_status in create_tasks_reply.creation_status_list.creation_statuses:
                if creation_status.WhichOneof("Status") == "task_info":
                    tasks_created.append(
                        Task(
                            id=creation_status.task_info.task_id,
                            session_id=self.session_id,
                            expected_output_ids=[
                                k for k in creation_status.task_info.expected_output_keys
                            ],
                            data_dependencies=[
                                k for k in creation_status.task_info.data_dependencies
                            ],
                        )
                    )
                else:
                    tasks_creation_failed.append(creation_status.error)
        else:
            raise Exception("Unknown value")
        return tasks_created, tasks_creation_failed

    def submit_tasks(
        self,
        tasks: List[TaskDefinition],
        default_task_options: Optional[TaskOptions] = None,
        batch_size: Optional[int] = 100,
    ) -> None:
        """Submit tasks to the agent.

        Args:
            tasks: List of task definitions
            default_task_options: Default Task Options used if a task has its options not set
            batch_size: Batch size for submission
        """
        for tasks_batch in batched(tasks, batch_size):
            task_creations = []

            for t in tasks_batch:
                task_creation = SubmitTasksRequest.TaskCreation(
                    expected_output_keys=t.expected_output_ids,
                    payload_id=t.payload_id,
                    data_dependencies=t.data_dependencies,
                )
                if t.options:
                    task_creation.task_options = t.options.to_message()
                task_creations.append(task_creation)

            request = SubmitTasksRequest(
                session_id=self.session_id,
                communication_token=self.token,
                task_creations=task_creations,
            )

            if default_task_options:
                request.task_options = (default_task_options.to_message(),)

            self._client.SubmitTasks(request)

    def send_results(self, results_data: Dict[str, bytes | bytearray]) -> None:
        """Send results.

        Args:
            result_data: A dictionnary mapping each result ID to its data.
        """
        for result_id, result_data in results_data.items():
            with open(os.path.join(self.data_folder, result_id), "wb") as f:
                f.write(result_data)

        request = NotifyResultDataRequest(
            ids=[
                NotifyResultDataRequest.ResultIdentifier(
                    session_id=self.session_id, result_id=result_id
                )
                for result_id in results_data.keys()
            ],
            communication_token=self.token,
        )
        self._client.NotifyResultData(request)

    def create_results_metadata(
        self, result_names: List[str], batch_size: int = 100
    ) -> Dict[str, List[Result]]:
        """
        Create the metadata of multiple results at once.
        Data have to be uploaded separately.

        Args:
            result_names: The names of the results to create.
            batch_size: Batch size for querying.

        Return:
            A dictionnary mapping each result name to its result summary.
        """
        results = {}
        for result_names_batch in batched(result_names, batch_size):
            request = CreateResultsMetaDataRequest(
                results=[
                    CreateResultsMetaDataRequest.ResultCreate(name=result_name)
                    for result_name in result_names
                ],
                session_id=self.session_id,
                communication_token=self.token,
            )
            response: CreateResultsMetaDataResponse = self._client.CreateResultsMetaData(request)
            for result_message in response.results:
                results[result_message.name] = Result.from_message(result_message)
        return results

    def create_results(
        self, results_data: Dict[str, bytes], batch_size: int = 1
    ) -> Dict[str, Result]:
        """Create one result with data included in the request.

        Args:
            results_data: A dictionnary mapping the result names to their actual data.
            batch_size: Batch size for querying.

        Return:
            A dictionnary mappin each result name to its corresponding result summary.
        """
        results = {}
        for results_ids_batch in batched(results_data.keys(), batch_size):
            request = CreateResultsRequest(
                results=[
                    CreateResultsRequest.ResultCreate(name=name, data=results_data[name])
                    for name in results_ids_batch
                ],
                session_id=self.session_id,
                communication_token=self.token,
            )
            response: CreateResultsResponse = self._client.CreateResults(request)
            for message in response.results:
                results[message.name] = Result.from_message(message)
        return results


def _to_request_stream_internal(request, communication_token, is_last, chunk_max_size):
    req = CreateTaskRequest(
        init_task=InitTaskRequest(
            header=TaskRequestHeader(
                data_dependencies=request.data_dependencies,
                expected_output_keys=request.expected_output_keys,
            )
        ),
        communication_token=communication_token,
    )
    yield req
    start = 0
    payload_length = len(request.payload)
    if payload_length == 0:
        req = CreateTaskRequest(
            task_payload=DataChunk(data=b""), communication_token=communication_token
        )
        yield req
    while start < payload_length:
        chunk_size = min(chunk_max_size, payload_length - start)
        req = CreateTaskRequest(
            task_payload=DataChunk(data=request.payload[start : start + chunk_size]),
            communication_token=communication_token,
        )
        yield req
        start += chunk_size
    req = CreateTaskRequest(
        task_payload=DataChunk(data_complete=True),
        communication_token=communication_token,
    )
    yield req

    if is_last:
        req = CreateTaskRequest(
            init_task=InitTaskRequest(last_task=True),
            communication_token=communication_token,
        )
        yield req


def _to_request_stream(requests, communication_token, t_options, chunk_max_size):
    if t_options is None:
        req = CreateTaskRequest(
            init_request=CreateTaskRequest.InitRequest(),
            communication_token=communication_token,
        )
    else:
        req = CreateTaskRequest(
            init_request=CreateTaskRequest.InitRequest(task_options=t_options),
            communication_token=communication_token,
        )
    yield req
    if len(requests) == 0:
        return
    for r in requests[:-1]:
        for req in _to_request_stream_internal(r, communication_token, False, chunk_max_size):
            yield req
    for req in _to_request_stream_internal(requests[-1], communication_token, True, chunk_max_size):
        yield req



from __future__ import annotations
import traceback
from concurrent import futures
from typing import Callable, Union

import grpc
from grpc import Channel

from .seqlogger import ClefLogger
from ..common import Output, HealthCheckStatus
from ..protogen.common.objects_pb2 import Empty
from ..protogen.common.worker_common_pb2 import (
    ProcessReply,
    ProcessRequest,
    HealthCheckReply,
)
from ..protogen.worker.agent_service_pb2_grpc import AgentStub
from ..protogen.worker.worker_service_pb2_grpc import (
    WorkerServicer,
    add_WorkerServicer_to_server,
)
from .taskhandler import TaskHandler


class ArmoniKWorker(WorkerServicer):
    def __init__(
        self,
        agent_channel: Channel,
        processing_function: Callable[[TaskHandler], Output],
        health_check: Callable[
            [], HealthCheckReply.ServingStatus
        ] = lambda: HealthCheckStatus.SERVING,
        logger=ClefLogger.getLogger("ArmoniKWorker"),
    ):
        """Creates a worker for ArmoniK

        Args:
            agent_channel: gRPC channel for the agent
            processing_function: Function that will be called when a
                task needs processing
            health_check: Function that will be called to check the
                health of the worker. Defaults to a simple "Serving"
                reply
            logger: Logger used for the worker, defaults to a logger
                ArmoniKWorker compatible with Seq
        """
        self.health_check = health_check
        self.processing_function = processing_function
        self._client = AgentStub(agent_channel)
        self._logger = logger

    def start(self, endpoint: str):
        """Starts the worker

        Args:
            endpoint: endpoint from which to listen to requests
        """
        server = grpc.server(futures.ThreadPoolExecutor(max_workers=1))
        add_WorkerServicer_to_server(self, server)
        server.add_insecure_port(endpoint)
        server.start()
        server.wait_for_termination()

    def Process(self, request: ProcessRequest, context) -> Union[ProcessReply, None]:
        try:
            self._logger.debug("Received task")
            task_handler = TaskHandler(request, self._client)
            return ProcessReply(output=self.processing_function(task_handler).to_message())
        except Exception as e:
            self._logger.exception(
                f"Failed task {''.join(traceback.format_exception(type(e) ,e, e.__traceback__))}",
                exc_info=e,
            )

    def HealthCheck(self, request: Empty, context) -> HealthCheckReply:
        return HealthCheckReply(status=self.health_check())



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from . import objects_pb2 as _objects_pb2
from . import result_status_pb2 as _result_status_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class CreateTaskRequest(_message.Message):
    __slots__ = ["init_request", "init_task", "task_payload", "communication_token"]
    class InitRequest(_message.Message):
        __slots__ = ["task_options"]
        TASK_OPTIONS_FIELD_NUMBER: _ClassVar[int]
        task_options: _objects_pb2.TaskOptions
        def __init__(self, task_options: _Optional[_Union[_objects_pb2.TaskOptions, _Mapping]] = ...) -> None: ...
    INIT_REQUEST_FIELD_NUMBER: _ClassVar[int]
    INIT_TASK_FIELD_NUMBER: _ClassVar[int]
    TASK_PAYLOAD_FIELD_NUMBER: _ClassVar[int]
    COMMUNICATION_TOKEN_FIELD_NUMBER: _ClassVar[int]
    init_request: CreateTaskRequest.InitRequest
    init_task: _objects_pb2.InitTaskRequest
    task_payload: _objects_pb2.DataChunk
    communication_token: str
    def __init__(self, init_request: _Optional[_Union[CreateTaskRequest.InitRequest, _Mapping]] = ..., init_task: _Optional[_Union[_objects_pb2.InitTaskRequest, _Mapping]] = ..., task_payload: _Optional[_Union[_objects_pb2.DataChunk, _Mapping]] = ..., communication_token: _Optional[str] = ...) -> None: ...

class CreateTaskReply(_message.Message):
    __slots__ = ["creation_status_list", "error", "communication_token"]
    class TaskInfo(_message.Message):
        __slots__ = ["task_id", "expected_output_keys", "data_dependencies", "payload_id"]
        TASK_ID_FIELD_NUMBER: _ClassVar[int]
        EXPECTED_OUTPUT_KEYS_FIELD_NUMBER: _ClassVar[int]
        DATA_DEPENDENCIES_FIELD_NUMBER: _ClassVar[int]
        PAYLOAD_ID_FIELD_NUMBER: _ClassVar[int]
        task_id: str
        expected_output_keys: _containers.RepeatedScalarFieldContainer[str]
        data_dependencies: _containers.RepeatedScalarFieldContainer[str]
        payload_id: str
        def __init__(self, task_id: _Optional[str] = ..., expected_output_keys: _Optional[_Iterable[str]] = ..., data_dependencies: _Optional[_Iterable[str]] = ..., payload_id: _Optional[str] = ...) -> None: ...
    class CreationStatus(_message.Message):
        __slots__ = ["task_info", "error"]
        TASK_INFO_FIELD_NUMBER: _ClassVar[int]
        ERROR_FIELD_NUMBER: _ClassVar[int]
        task_info: CreateTaskReply.TaskInfo
        error: str
        def __init__(self, task_info: _Optional[_Union[CreateTaskReply.TaskInfo, _Mapping]] = ..., error: _Optional[str] = ...) -> None: ...
    class CreationStatusList(_message.Message):
        __slots__ = ["creation_statuses"]
        CREATION_STATUSES_FIELD_NUMBER: _ClassVar[int]
        creation_statuses: _containers.RepeatedCompositeFieldContainer[CreateTaskReply.CreationStatus]
        def __init__(self, creation_statuses: _Optional[_Iterable[_Union[CreateTaskReply.CreationStatus, _Mapping]]] = ...) -> None: ...
    CREATION_STATUS_LIST_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    COMMUNICATION_TOKEN_FIELD_NUMBER: _ClassVar[int]
    creation_status_list: CreateTaskReply.CreationStatusList
    error: str
    communication_token: str
    def __init__(self, creation_status_list: _Optional[_Union[CreateTaskReply.CreationStatusList, _Mapping]] = ..., error: _Optional[str] = ..., communication_token: _Optional[str] = ...) -> None: ...

class DataRequest(_message.Message):
    __slots__ = ["communication_token", "result_id"]
    COMMUNICATION_TOKEN_FIELD_NUMBER: _ClassVar[int]
    RESULT_ID_FIELD_NUMBER: _ClassVar[int]
    communication_token: str
    result_id: str
    def __init__(self, communication_token: _Optional[str] = ..., result_id: _Optional[str] = ...) -> None: ...

class DataResponse(_message.Message):
    __slots__ = ["result_id"]
    RESULT_ID_FIELD_NUMBER: _ClassVar[int]
    result_id: str
    def __init__(self, result_id: _Optional[str] = ...) -> None: ...

class CreateResultsMetaDataRequest(_message.Message):
    __slots__ = ["results", "session_id", "communication_token"]
    class ResultCreate(_message.Message):
        __slots__ = ["name"]
        NAME_FIELD_NUMBER: _ClassVar[int]
        name: str
        def __init__(self, name: _Optional[str] = ...) -> None: ...
    RESULTS_FIELD_NUMBER: _ClassVar[int]
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    COMMUNICATION_TOKEN_FIELD_NUMBER: _ClassVar[int]
    results: _containers.RepeatedCompositeFieldContainer[CreateResultsMetaDataRequest.ResultCreate]
    session_id: str
    communication_token: str
    def __init__(self, results: _Optional[_Iterable[_Union[CreateResultsMetaDataRequest.ResultCreate, _Mapping]]] = ..., session_id: _Optional[str] = ..., communication_token: _Optional[str] = ...) -> None: ...

class ResultMetaData(_message.Message):
    __slots__ = ["session_id", "result_id", "name", "status", "created_at"]
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    RESULT_ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    session_id: str
    result_id: str
    name: str
    status: _result_status_pb2.ResultStatus
    created_at: _timestamp_pb2.Timestamp
    def __init__(self, session_id: _Optional[str] = ..., result_id: _Optional[str] = ..., name: _Optional[str] = ..., status: _Optional[_Union[_result_status_pb2.ResultStatus, str]] = ..., created_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class CreateResultsMetaDataResponse(_message.Message):
    __slots__ = ["results", "communication_token"]
    RESULTS_FIELD_NUMBER: _ClassVar[int]
    COMMUNICATION_TOKEN_FIELD_NUMBER: _ClassVar[int]
    results: _containers.RepeatedCompositeFieldContainer[ResultMetaData]
    communication_token: str
    def __init__(self, results: _Optional[_Iterable[_Union[ResultMetaData, _Mapping]]] = ..., communication_token: _Optional[str] = ...) -> None: ...

class SubmitTasksRequest(_message.Message):
    __slots__ = ["session_id", "task_options", "task_creations", "communication_token"]
    class TaskCreation(_message.Message):
        __slots__ = ["expected_output_keys", "data_dependencies", "payload_id", "task_options"]
        EXPECTED_OUTPUT_KEYS_FIELD_NUMBER: _ClassVar[int]
        DATA_DEPENDENCIES_FIELD_NUMBER: _ClassVar[int]
        PAYLOAD_ID_FIELD_NUMBER: _ClassVar[int]
        TASK_OPTIONS_FIELD_NUMBER: _ClassVar[int]
        expected_output_keys: _containers.RepeatedScalarFieldContainer[str]
        data_dependencies: _containers.RepeatedScalarFieldContainer[str]
        payload_id: str
        task_options: _objects_pb2.TaskOptions
        def __init__(self, expected_output_keys: _Optional[_Iterable[str]] = ..., data_dependencies: _Optional[_Iterable[str]] = ..., payload_id: _Optional[str] = ..., task_options: _Optional[_Union[_objects_pb2.TaskOptions, _Mapping]] = ...) -> None: ...
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    TASK_OPTIONS_FIELD_NUMBER: _ClassVar[int]
    TASK_CREATIONS_FIELD_NUMBER: _ClassVar[int]
    COMMUNICATION_TOKEN_FIELD_NUMBER: _ClassVar[int]
    session_id: str
    task_options: _objects_pb2.TaskOptions
    task_creations: _containers.RepeatedCompositeFieldContainer[SubmitTasksRequest.TaskCreation]
    communication_token: str
    def __init__(self, session_id: _Optional[str] = ..., task_options: _Optional[_Union[_objects_pb2.TaskOptions, _Mapping]] = ..., task_creations: _Optional[_Iterable[_Union[SubmitTasksRequest.TaskCreation, _Mapping]]] = ..., communication_token: _Optional[str] = ...) -> None: ...

class SubmitTasksResponse(_message.Message):
    __slots__ = ["task_infos", "communication_token"]
    class TaskInfo(_message.Message):
        __slots__ = ["task_id", "expected_output_ids", "data_dependencies", "payload_id"]
        TASK_ID_FIELD_NUMBER: _ClassVar[int]
        EXPECTED_OUTPUT_IDS_FIELD_NUMBER: _ClassVar[int]
        DATA_DEPENDENCIES_FIELD_NUMBER: _ClassVar[int]
        PAYLOAD_ID_FIELD_NUMBER: _ClassVar[int]
        task_id: str
        expected_output_ids: _containers.RepeatedScalarFieldContainer[str]
        data_dependencies: _containers.RepeatedScalarFieldContainer[str]
        payload_id: str
        def __init__(self, task_id: _Optional[str] = ..., expected_output_ids: _Optional[_Iterable[str]] = ..., data_dependencies: _Optional[_Iterable[str]] = ..., payload_id: _Optional[str] = ...) -> None: ...
    TASK_INFOS_FIELD_NUMBER: _ClassVar[int]
    COMMUNICATION_TOKEN_FIELD_NUMBER: _ClassVar[int]
    task_infos: _containers.RepeatedCompositeFieldContainer[SubmitTasksResponse.TaskInfo]
    communication_token: str
    def __init__(self, task_infos: _Optional[_Iterable[_Union[SubmitTasksResponse.TaskInfo, _Mapping]]] = ..., communication_token: _Optional[str] = ...) -> None: ...

class CreateResultsRequest(_message.Message):
    __slots__ = ["results", "session_id", "communication_token"]
    class ResultCreate(_message.Message):
        __slots__ = ["name", "data"]
        NAME_FIELD_NUMBER: _ClassVar[int]
        DATA_FIELD_NUMBER: _ClassVar[int]
        name: str
        data: bytes
        def __init__(self, name: _Optional[str] = ..., data: _Optional[bytes] = ...) -> None: ...
    RESULTS_FIELD_NUMBER: _ClassVar[int]
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    COMMUNICATION_TOKEN_FIELD_NUMBER: _ClassVar[int]
    results: _containers.RepeatedCompositeFieldContainer[CreateResultsRequest.ResultCreate]
    session_id: str
    communication_token: str
    def __init__(self, results: _Optional[_Iterable[_Union[CreateResultsRequest.ResultCreate, _Mapping]]] = ..., session_id: _Optional[str] = ..., communication_token: _Optional[str] = ...) -> None: ...

class CreateResultsResponse(_message.Message):
    __slots__ = ["results", "communication_token"]
    RESULTS_FIELD_NUMBER: _ClassVar[int]
    COMMUNICATION_TOKEN_FIELD_NUMBER: _ClassVar[int]
    results: _containers.RepeatedCompositeFieldContainer[ResultMetaData]
    communication_token: str
    def __init__(self, results: _Optional[_Iterable[_Union[ResultMetaData, _Mapping]]] = ..., communication_token: _Optional[str] = ...) -> None: ...

class NotifyResultDataRequest(_message.Message):
    __slots__ = ["ids", "communication_token"]
    class ResultIdentifier(_message.Message):
        __slots__ = ["session_id", "result_id"]
        SESSION_ID_FIELD_NUMBER: _ClassVar[int]
        RESULT_ID_FIELD_NUMBER: _ClassVar[int]
        session_id: str
        result_id: str
        def __init__(self, session_id: _Optional[str] = ..., result_id: _Optional[str] = ...) -> None: ...
    IDS_FIELD_NUMBER: _ClassVar[int]
    COMMUNICATION_TOKEN_FIELD_NUMBER: _ClassVar[int]
    ids: _containers.RepeatedCompositeFieldContainer[NotifyResultDataRequest.ResultIdentifier]
    communication_token: str
    def __init__(self, ids: _Optional[_Iterable[_Union[NotifyResultDataRequest.ResultIdentifier, _Mapping]]] = ..., communication_token: _Optional[str] = ...) -> None: ...

class NotifyResultDataResponse(_message.Message):
    __slots__ = ["result_ids"]
    RESULT_IDS_FIELD_NUMBER: _ClassVar[int]
    result_ids: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, result_ids: _Optional[_Iterable[str]] = ...) -> None: ...
